# ---
# jupyter:
#   jupytext:
#     formats: ipynb,py:percent
#     text_representation:
#       extension: .py
#       format_name: percent
#       format_version: '1.3'
#       jupytext_version: 1.11.4
#   kernelspec:
#     display_name: Python 3 (ipykernel)
#     language: python
#     name: python3
# ---

# %% [markdown]
# ## Preprocess model data: 

# %%
from pathlib import Path
import matplotlib as mpl
import xarray as xr


# %%
from IPython import get_ipython

# noinspection PyBroadException
try:
    _ipython = get_ipython()
    _magic = _ipython.magic
    _magic('load_ext autoreload')
    _magic('autoreload 2')
except:
    pass

# %%
from pathlib import Path

from bs_fdbck_clean.util.BSOA_datamanip import ds2df_inc_preprocessing
from bs_fdbck_clean.util.collocate.collocateLONLAToutput import CollocateLONLATout
from bs_fdbck_clean.util.collocate.collocate_echam_salsa import CollocateModelEcham
import useful_scit.util.log as log

from bs_fdbck_clean.util.plot.BSOA_plots import make_cool_grid, plot_scatter

import time
import xarray as xr
import matplotlib.pyplot as plt

# %%
import seaborn as sns
import matplotlib.pyplot as plt

# %%
import numpy as np

# %%
import numpy as np
from sklearn.linear_model import LinearRegression, BayesianRidge

# %%
from bs_fdbck_clean.constants import path_measurement_data
import matplotlib.pyplot as plt

# %%
import pandas as pd
import numpy as np
import xarray as xr
xr.set_options(keep_attrs=True)

# %%
import datetime 

# %%
# %load_ext autoreload
# %autoreload 2

# %% [markdown] tags=[]
# ## Read in model data. 

# %%
model_lev_i=-2
select_station = 'ATTO'

# %% [markdown]
# #### Model level different in UKESM because model has much finer resolution close to the ground 

# %%
model_lev_i_ukesm = -2

# %%
postproc_data = path_measurement_data /'model_station'/select_station
postproc_data.mkdir(parents=True, exist_ok=True)

# %%
models = ['ECHAM-SALSA','NorESM', 'EC-Earth', 'UKESM']

di_mod2cases = dict()
#for mod in models:
#    di_mod2cases[mod]=dict()

# %% [markdown] tags=[]
# #### Define dictionaries:

# %%
dic_mod_ca = dict()
dic_df_mod_case = dict()
dic_dfsm_mod_case = dict()

# %% [markdown]
# ### ECHAM SALSA:

# %%
from_t = '2012-01-01'
to_t = '2019-01-01'


# %%
case_name = 'SALSA_BSOA_feedback'
case_name_echam = 'SALSA_BSOA_feedback'
time_res = 'hour'
space_res='locations'
model_name='ECHAM-SALSA'


case_mod = case_name#'OsloAero_intBVOC_f19_f19_mg17_fssp'
cases_echam = [case_name]
di_mod2cases[model_name]=cases_echam

# %% [markdown]
# #### Variables

# %%
varl =[
    'apm',
    'geom',
    'airdens',
    'tempair',
    'uw',
    'vw',
    'ccn02',
    'ccn10',
    'cod',
    'cwp',
    'ceff',
    'ceff_ct',
    'lcdnc',
    'lcdnc_ct',
    'clfr',
    'cl_time',
    'aot550nm',
    'aot865nm',
    'ang550865',
    'up_sw',
    'up_sw_cs',
    'up_sw_noa',
    'up_sw_cs_noa',
    'up_lw',
    'up_lw_cs',
    'up_lw_noa',
    'up_lw_cs_noa',
    'mmrtrN500',
    'mmrtrN250',
    'mmrtrN200',
    'mmrtrN100',
    'mmrtrN50',
    'mmrtrN3',
    'oh_con',
    'emi_monot_bio',
    'emi_isop_bio',
    'SO2_gas',
    'APIN_gas',
    'TBETAOCI_gas',
    'BPIN_gas',
    'LIMON_gas',
    'SABIN_gas',
    'MYRC_gas',
    'CARENE3_gas',
    'ISOP_gas',
    'VBS0_gas',
    'VBS1_gas',
    'VBS10_gas',
    'ORG_mass',
] 


# %% [markdown] tags=[]
# #### Read in ECHAM-SALSA:

# %% tags=[]
for case_name in cases_echam:
    varlist = varl
    c = CollocateLONLATout(case_name, from_t, to_t,
                           True,
                           'hour',
                           model_name=model_name
                          # history_field=history_field
                          )
    if c.check_if_load_raw_necessary(varlist ):
        time1 = time.time()
        a = c.make_station_data_merge_monthly(varlist)
        print(a)

        time2 = time.time()
        print('DONE : took {:.3f} s'.format( (time2-time1)))
    else:
        print('UUUPS')
# %% tags=[]
dic_ds = dict()
for ca in cases_echam:
    c = CollocateLONLATout(ca, from_t, to_t,
                           True,
                           'hour',
                           model_name=model_name
                          )
                          # history_field=history_field)
    ds = c.get_collocated_dataset(varl)
    if 'location' in ds.coords:
        ds = ds.rename({'location':'station'})
    dic_ds[ca]=ds.drop('station').rename(dict(locations='station'))

# %%
dic_mod_ca['ECHAM-SALSA'] = dic_ds.copy()

# %% [markdown]
# ### EC-Earth

# %%
from_t = '2012-01-01'
to_t = '2019-01-01'


# %%
case_name = 'ECE3_output_Sara'
case_name_ec_earth = 'ECE3_output_Sara'
from_time = '2012-01'
to_time = '2019-02'
time_res = 'hour'
space_res='locations'
model_name='EC-Earth'


case_mod = case_name#'OsloAero_intBVOC_f19_f19_mg17_fssp'
cases_ec_earth = [case_name]
di_mod2cases[model_name]=cases_ec_earth

# %% [markdown]
# #### Variables

# %%
varl =[
    'CCN0.20',
    'CCN1.00',
    'M_SO4NUS',
    'M_SOANUS',
    'M_BCAIS',
    'M_POMAIS',
    'M_SOAAIS',
    'M_SO4ACS',
    'M_BCACS',
    'M_POMACS',
    'M_SSACS',
    'M_DUACS',
    'M_SOAACS',
    'M_SO4COS',
    'M_BCCOS',
    'M_POMCOS',
    'M_SSCOS',
    'M_DUCOS',
    'M_SOACOS',
    'M_BCAII',
    'M_POMAII',
    'M_SOAAII',
    'M_DUACI',
    'M_DUCOI',
    'N_NUS',
    'N_AIS',
    'N_ACS',
    'N_COS',
    'N_AII',
    'N_ACI',
    'N_COI',
    'RWET_NUS',
    'RWET_AIS',
    'RWET_ACS',
    'RWET_COS',
    'RWET_AII',
    'RWET_ACI',
    'RWET_COI',
    'RDRY_NUS',
    'RDRY_AIS',
    'RDRY_ACS',
    'RDRY_COS',
    'od550aer',
    'od550oa',
    'od550soa',
    'od440aer',
    'od870aer',
    'od350aer',
    'loadsoa',
    'emiterp',
    'emiisop',
    
]
varl_ifs=[
    'var130',
    'var176',

]


# %% [markdown] tags=[]
# #### Read in EC-Earth:

# %% tags=[]
for case_name in cases_ec_earth:
    varlist = varl
    c = CollocateLONLATout(case_name, from_t, to_t,
                           True,
                           'hour',
                           model_name=model_name
                          # history_field=history_field
                          )
    if c.check_if_load_raw_necessary(varlist ):
        time1 = time.time()
        a = c.make_station_data_merge_monthly(varlist)
        print(a)

        time2 = time.time()
        print('DONE : took {:.3f} s'.format( (time2-time1)))
    else:
        print('UUUPS')
# %%
from bs_fdbck_clean.util.BSOA_datamanip.ec_earth import fix_timestamp_ec_earth

# %% tags=[]
dic_ds = dict()
for ca in cases_ec_earth:
    c = CollocateLONLATout(ca, from_t, to_t,
                           True,
                           'hour',
                           model_name=model_name
                          )
                          # history_field=history_field)


    ds_ifs = c.get_collocated_dataset(varl_ifs)


    ds = c.get_collocated_dataset(varl)
    ds = fix_timestamp_ec_earth(ds)
    ds_ifs = ds_ifs.rename({'lev':'lev_ifs'})


    ds = xr.merge([ds,ds_ifs])
    if 'location' in ds.coords:
        ds = ds.rename({'location':'station'})
    dic_ds[ca]=ds.copy()

# %%
dic_mod_ca['EC-Earth'] = dic_ds.copy()

# %%
dic_mod_ca['EC-Earth']['ECE3_output_Sara']

# %%
# %% [markdown] tags=[]
# ### UKESM

# %%
case_name_ukesm = 'AEROCOMTRAJ'
case_name = case_name_ukesm
time_res = 'hour'
space_res = 'locations'
model_name = 'UKESM'

from_t = '2012-01-01'
to_t = '2019-01-01'


# %% tags=[]
case_mod = case_name
cases_ukesm = [case_name]
di_mod2cases[model_name]=cases_ukesm

# %% [markdown]
# #### Variables

# %%
varl = [
    'Mass_Conc_OM_NS',
    'Mass_Conc_OM_KS',
    'Mass_Conc_OM_KI',
    'Mass_Conc_OM_AS',
    'Mass_Conc_OM_CS',
    'mmrtr_OM_NS',
    'mmrtr_OM_KS',
    'mmrtr_OM_KI',
    'mmrtr_OM_AS',
    'mmrtr_OM_CS',
    'nconcNS',
    'nconcKS',
    'nconcKI',
    'nconcAS',
    'nconcCS',
    'ddryNS',
    'ddryKS',
    'ddryKI',
    'ddryAS',
    'ddryCS',
    'Temp',
    'SFisoprene',
    'SFterpene',

]


# %% [markdown] tags=[]
# #### Read in UKESM:

# %% tags=[]
for case_name in cases_ukesm:
    varlist = varl
    c = CollocateLONLATout(case_name, from_t, to_t,
                           True,
                           'hour',
                           model_name=model_name
                           # history_field=history_field
                           )
    if c.check_if_load_raw_necessary(varlist ):
        time1 = time.time()
        a = c.make_station_data_merge_monthly(varlist)
        print(a)

        time2 = time.time()
        print('DONE : took {:.3f} s'.format( (time2-time1)))
    else:
        print('UUUPS')
# %% tags=[]
dic_ds = dict()
for ca in cases_ukesm:
    c = CollocateLONLATout(ca, from_t, to_t,
                           True,
                           'hour',
                           model_name=model_name
                           )
    # history_field=history_field)




    ds = c.get_collocated_dataset(varl)


    if 'location' in ds.coords:
        ds = ds.rename({'location':'station'})
    ds = ds.rename({'model_level':'lev'})
    dic_ds[ca]=ds.copy()

# %%
dic_mod_ca['UKESM'] = dic_ds.copy()

# %%
_ds = dic_mod_ca['UKESM'][case_name_ukesm].sel(station='SMR')

# %%

# %%
_ds['hour'] = _ds['time.hour']
_ds

# %%
f, ax = plt.subplots()
_ds['nconcNS'].groupby(_ds['hour']).mean().isel(lev= 0).plot(ax = ax)
ax2 = ax.twinx()
_ds['nconcAS'].groupby(_ds['hour']).mean().isel(lev= 0).plot(ax = ax2, c='r')


# %% [markdown]
# ### NORESM

# %% [markdown]
# #### Settings:

# %%
nr_of_bins = 5
maxDiameter = 39.6  #    23.6 #e-9
minDiameter = 5.0  # e-9
history_field='.h1.'

# %%
from_t = '2012-01-01'
to_t = '2015-01-01'

# %%
from_t2 = '2015-01-01'
to_t2 = '2019-01-01'

# %% [markdown] tags=[]
# #### Cases:

# %%
cases_noresm1 = ['OsloAero_intBVOC_f09_f09_mg17_full']
cases_noresm2 = ['OsloAero_intBVOC_f09_f09_mg17_ssp245']
# %%
case_mod = 'OsloAero_intBVOC_f09_f09_mg17_fssp'
case_noresm = 'OsloAero_intBVOC_f09_f09_mg17_fssp'
cases_noresm = [case_noresm]
di_mod2cases['NorESM'] = cases_noresm

# %% [markdown] tags=[]
# #### Variables

# %%
varl =['N100','SOA_NA','SOA_A1','SO4_NA','DOD500','DOD440','ACTREL',#'TGCLDLWP',
       'H2SO4','SOA_LV','COAGNUCL','FORMRATE','FSNSC',
       'NUCLRATE','NCONC01','NCONC02','NCONC03','NCONC04','NCONC05','NCONC06','NCONC07',
       'NCONC08','NCONC09','NCONC10','NCONC11','NCONC12','NCONC13','NCONC14','SIGMA01',
       'SIGMA02','SIGMA03','SIGMA04','SIGMA05','SIGMA06','SIGMA07','SIGMA08','SIGMA09',
       'SIGMA10','SIGMA11','SIGMA12','SIGMA13','SIGMA14','NMR01','NMR02','NMR03','NMR04',
       'NMR05','NMR06','NMR07','NMR08','NMR09','NMR10','NMR11','NMR12','NMR13','NMR14', 
      'FSNS','FSDS_DRF','T','GR','GRH2SO4','GRSOA','TGCLDCWP','U','V', 'SO2','isoprene',
       'monoterp','GS_SO2', 'GS_H2SO4','GS_monoterp','GS_isoprene']


varl =['N100','DOD500','DOD440','ACTREL',#,'SOA_A1',
       'H2SO4','SOA_LV','COAGNUCL','FORMRATE','T','SOA_SV',
       'NCONC01','N50','N150','N200',#'DOD500',
       #'DOD500',
      'isoprene',
      'SFisoprene',
       'monoterp',
       'SFmonoterp',
       'PS',
       'NNAT_0',
       #'NCONC00',
       'NCONC02',
       'NCONC03',
       'NCONC04',
       'NCONC05',
       'NCONC06',
       'NCONC07',
       'NCONC08',
       'NCONC09',
       'NCONC10',
       'NCONC11',
       'NCONC12',
       'NCONC13',
       'NCONC14',
       'NMR01',
       #'NMR00',
       #'SIGMA00',
       'SIGMA01',
       'NMR02',
       'SIGMA02',
       'NMR03',
       'SIGMA03',
       'NMR04',
       'SIGMA04',
       'NMR05',
       'SIGMA05',
       'NMR06',
       'SIGMA06',
       'NMR07',
       'SIGMA07',
       'NMR08',
       'SIGMA08',
       'NMR09',
       'SIGMA09',
       'NMR10',
       'SIGMA10',
       'NMR11',
       'SIGMA11',
       'NMR12',
       'SIGMA12',
       'NMR13',
       'SIGMA13',
       'NMR14',
       'SIGMA14',
       #'hyam','hybm',
       'FSNS','FSDS_DRF',
       'FSNSC',
       'FSDSCDRF',
       #'hyai','hybi',
       'N500',
      'SOA_NA','SOA_A1','OM_NI','OM_AI','OM_AC','SO4_NA','SO4_A1','SO4_A2','SO4_AC','SO4_PR',
      'BC_N','BC_AX','BC_NI','BC_A','BC_AI','BC_AC','SS_A1','SS_A2','SS_A3','DST_A2','DST_A3', 
      ] 


# %% [markdown]
# #### Read in NorESM

# %% tags=[]
for case_name in cases_noresm1:
    varlist = varl
    c = CollocateLONLATout(case_name, from_t, to_t,
                           True,
                           'hour',
                           history_field=history_field)
    if c.check_if_load_raw_necessary(varlist ):
        time1 = time.time()
        a = c.make_station_data_merge_monthly(varlist)
        print(a)

        time2 = time.time()
        print('DONE : took {:.3f} s'.format( (time2-time1)))
    else:
        print('UUUPS')
# %% tags=[]
for case_name in cases_noresm2:
    varlist = varl# list_sized_vars_noresm
    c = CollocateLONLATout(case_name, from_t2, to_t2,
                           False,
                           'hour',
                           history_field=history_field)
    if c.check_if_load_raw_necessary(varlist ):
        time1 = time.time()
        a = c.make_station_data_merge_monthly(varlist)
        print(a)

        time2 = time.time()
        print('DONE : took {:.3f} s'.format( (time2-time1)))
    else:
        print('UUUPS')

# %% tags=[]
dic_ds = dict()
for ca in cases_noresm1:
    c = CollocateLONLATout(ca, from_t, to_t,
                           False,
                           'hour',
                           history_field=history_field)
    ds = c.get_collocated_dataset(varl)
    #ds2 = c.get_collocated_dataset(['DOD500'])
    if 'location' in ds.coords:
        ds = ds.rename({'location':'station'})
    dic_ds[ca]=ds

# %% tags=[]
#dic_ds = dict()
for ca in cases_noresm2:
    c = CollocateLONLATout(ca, from_t2, to_t2,
                           False,
                           'hour',
                           history_field=history_field)
    ds = c.get_collocated_dataset(varl)
    if 'location' in ds.coords:
        ds = ds.rename({'location':'station'})
    dic_ds[ca]=ds

# %%
case1 = cases_noresm1[0]
case2 = cases_noresm2[0]

ds1 = dic_ds[case1]
ds2 = dic_ds[case2]

st_y = from_t.split('-')[0]
mid_y_t = str(int(to_t.split('-')[0])-1)
mid_y_f = to_t.split('-')[0]
end_y = to_t2.split('-')[0]

print(st_y, mid_y_t, mid_y_f, end_y)

# %%
_ds1 = ds1.sel(time=slice(st_y, mid_y_t))
_ds2 = ds2.sel(time=slice(mid_y_f, end_y))
ds_comb = xr.concat([_ds1, _ds2], dim='time')#.sortby('time')

# %%
dic_ds = {case_mod: ds_comb}


# %%
dic_mod_ca['NorESM'] = dic_ds.copy()

# %%
ds_comb['N50'].sel(station='ATTO').isel(lev=-1).plot()

# %%
vrlis = [f'NCONC{i:02}' for i in range(1,15)] +[f'NMR{i:02}' for i in range(1,15)] + [f'SIGMA{i:02}' for i in range(1,15)] 

# %%
from dask.diagnostics import ProgressBar

# %% [markdown] tags=[]
# ## SELECT STATION:
#

# %%
for mod in dic_mod_ca.keys():
    print(mod)
    for ca in dic_mod_ca[mod].keys():
        dic_mod_ca[mod][ca] = dic_mod_ca[mod][ca].sel(station=select_station)
        dic_mod_ca[mod][ca].load()

# %% [markdown] tags=[]
# ## Select level: 

# %%
for mod in dic_mod_ca.keys():
    if mod in ['EC-Earth', 'UKESM']:
        continue
    print(mod)
    for ca in dic_mod_ca[mod].keys():
        dic_mod_ca[mod][ca] = dic_mod_ca[mod][ca].isel(lev=model_lev_i)
        print(dic_mod_ca[mod][ca].lev)
        dic_mod_ca[mod][ca].load()

# %%
for mod in dic_mod_ca.keys():
    if mod not in ['EC-Earth', 'UKESM']:
        continue
    print(mod)
    if mod =='UKESM':
        print(f'Using level {model_lev_i_ukesm} for UKESM')
        lev_i_tm5 = -model_lev_i_ukesm-1
    else:
        lev_i_tm5 = -model_lev_i-1
    for ca in dic_mod_ca[mod].keys():
        dic_mod_ca[mod][ca] = dic_mod_ca[mod][ca].isel(lev=lev_i_tm5)
        if mod=='EC-Earth':
            dic_mod_ca[mod][ca] = dic_mod_ca[mod][ca].isel(lev_ifs=model_lev_i)
        dic_mod_ca[mod][ca].load()




# %% [markdown]
# TM5 has model levels going from the ground and up, IFS from the top and down.

# %% [markdown]
# ## Adjust ECHAM-SALSA

# %%
rn_dict_echam={
    'ORG_mass_conc' : 'OA',
    'tempair':'T',

    
}

# %%
from bs_fdbck_clean.util.BSOA_datamanip import calculate_daily_median_summer,calculate_summer_median, mask4summer,ds2df_echam

# %%

# %%
import datetime


# %% [markdown]
# ### Fix echam time

# %%
def fix_echam_time(dt):
    #a, b = divmod(round(dt.minute, -1), 60)
    tdelta = datetime.timedelta(minutes=dt.minute, seconds = dt.second)
    #nh = (dt.hour+a)%24
    ndt = datetime.datetime(dt.year, dt.month,dt.day, dt.hour)#dt - tdelta
    #dt_o = datetime.datetime(dt.year,dt.month, dt.day, (dt.hour + a) % 24,b)
    return ndt


# %%
dic_mod_ca['ECHAM-SALSA'][case_name_echam]

# %%





# %% [markdown]
# ### Run ds2df_echam

# %%
from bs_fdbck_clean.util.BSOA_datamanip import pressure_default, standard_air_density,R

df, df_sm = ds2df_echam(dic_mod_ca['ECHAM-SALSA'][case_name_echam], 
                        take_daily_median=False, 
                        air_density=standard_air_density,
                        model_lev_i =model_lev_i)
df.index = df.reset_index()['time'].apply(fix_echam_time)
df

# %%
df['N100'].plot()

# %% [markdown]
# ### Save result

# %%


_di = {case_name_echam:df}
_dism = {case_name_echam:df_sm}

dic_df_mod_case['ECHAM-SALSA']= _di.copy()
dic_dfsm_mod_case['ECHAM-SALSA'] = _dism.copy()

# %% [markdown]
# ## Adjust EC-Earth:

# %%
ds_ec_earth = dic_mod_ca['EC-Earth'][case_name_ec_earth]


# %% [markdown]
# #### Check all data there:

# %%
from bs_fdbck_clean.util.BSOA_datamanip.ec_earth import rad_vars, num_vars

# %%
for n in num_vars:
    ds_ec_earth[n].plot()
    plt.show()

# %%
for n in rad_vars:
    ds_ec_earth[n].plot()
    plt.show()

# %%
from bs_fdbck_clean.util.BSOA_datamanip.ec_earth import ds2df_ec_earth

# %%

df, df_sm = ds2df_ec_earth(ds_ec_earth,
                        take_daily_median=False,
                        #air_density=air_dens,
                        model_lev_i =model_lev_i)
#df.index = df.reset_index()['time'].apply(fix_echam_time)
df

# %% [markdown] tags=[]
# ### Assuming standard pressure for EC-Earth (since we don't have this and the effect is marginal)

# %% [markdown]
# Converting by:
#
# \begin{align}
# conc. OA_{amb}=\frac{m_{OA}}{V_{amb}} = & \frac{m_{OA}}{m_{air}} \cdot \frac{m_{air}}{V_{amb}} = w_{OA} \cdot \rho_{amb}
# \end{align}
# and in the same way
# \begin{align}
# conc. OA_{STP}= & \frac{m_{OA}}{m_{air}} \cdot \frac{m_{air}}{V_{STP}} = w_{OA} \cdot \rho_{STP}
# \end{align}
#
# So finally
# \begin{align}
# conc. OA_{STP}= & conc. OA_{amb} \cdot \frac{\rho_{STP}}{\rho_{amb}}
# \end{align}

# %%
from bs_fdbck_clean.util.BSOA_datamanip import pressure_default, standard_air_density,R

# %%
df['T_C']

# %%
df['T_K'] = df['T_C'] + 273.15

# %%
df_cp = df[['T_K','N50','OA']]

df_cp = df_cp.assign(hour=  df_cp.index.hour)

fig, ax = plt.subplots()
df_cp['OA'].groupby(df_cp['hour']).mean().plot(marker='o')
df_cp['T_K'].groupby(df_cp['hour']).mean().plot(marker='o', ax=ax.twinx(), c='r')
df_cp['T_K'].interpolate(method='quadratic').groupby(df_cp['hour']).mean().plot(marker='o', ax=ax.twinx(), c='m')

df_cp['T_K'].interpolate(method='linear').groupby(df_cp['hour']).mean().plot(marker='o', ax=ax.twinx(), c='y')

# %% [markdown]
# ## Interpolate temperature quadratically before calculating density. 

# %%
df['T_K_interp'] = df['T_K'].interpolate(method='quadratic', limit=2)

# %%
df['density'] = 1e5/(R*df['T_K_interp'])
df['density_no_interp'] = 1e5/(R*df['T_K'])

# %%
df['density'].plot(marker='.')
df['density_no_interp'].plot(marker='.')

# %%
df['density'].groupby(df.index.hour).mean().plot(marker='.')
df['density_no_interp'].groupby(df.index.hour).mean().plot(marker='.')

# %%
df['ambient2stp_correction_factor'] = (1/df['density'])*standard_air_density

# %%
vl = ['OA','N100','N200','N50','N500']
for v in vl:
    if v in df:
        df = df.rename({v:f'{v}_amb'}, axis=1)
        df[f'{v}_STP'] = df[f'{v}_amb']*df['ambient2stp_correction_factor']
        print(f'converting {v}')

# %%
df['T_C'].plot(marker='.')#.columns#['Temperature'].plot()

# %%
df['N50_STP'].plot.hist(alpha=0.6, bins=100, density=True)#.columns#['Temperature'].plot()
df['N50_amb'].plot.hist(alpha=0.6, bins=100, density=True)#.columns#['Temperature'].plot()

# %% [markdown]
# ### Save result

# %%


_di = {case_name_ec_earth:df}
_dism = {case_name_ec_earth:df_sm}

dic_df_mod_case['EC-Earth']= _di.copy()
dic_dfsm_mod_case['EC-Earth'] = _dism.copy()

# %% [markdown]
# ## Adjust UKESM:

# %%
ds_ukesm = dic_mod_ca['UKESM'][case_name_ukesm]


# %% [markdown]
# #### Check all data there:

# %%
from bs_fdbck_clean.util.BSOA_datamanip.ukesm import diam_vars, num_vars

# %% [markdown]
# ## Divide number concentrations by three: 
# The number concentrations have been erronously multiplied by three in this data, so we here divide by three to compensate for that

# %%
number_concentration_params_ukesm = ['nconcAS',
 'nconcCS',
 'nconcKI',
 'nconcKS',
 'nconcNS']

# %%
  
        
def add_scaling(_ds, nvar):
    _ds[nvar+'_uncorrected'] = _ds[nvar].copy()
    _ds[nvar] = (1/3)*_ds[nvar].copy()
    _ds[nvar].attrs['corrected_scaling'] = 'True'
    print(f'Dividing {nvar} by three')
    return _ds


for nvar in number_concentration_params_ukesm:
    print(nvar)
    if 'corrected_scaling' in ds_ukesm[nvar].attrs:
        if ds_ukesm[nvar].attrs['corrected_scaling']=='True':
            continue
        else:
            ds_ukesm = add_scaling(ds_ukesm, nvar)    
    else:
        ds_ukesm = add_scaling(ds_ukesm, nvar)
      

# %%
#ds_ukesm['nconcAS'].plot(marker='.')
ds_ukesm['nconcAS'].resample(time='10D').mean().plot(marker='.', alpha=.7)
ds_ukesm['nconcAS_uncorrected'].resample(time='10D').mean().plot(marker='.', alpha=.7)

# %% [markdown]
# #### Time step difference in UKESM

# %% [markdown]
# #### Adjust the emissions to be at the same timestep
# The emission dataset is labelled at the beginning of each time period, the rest is in the middle. We therefore need to adjust it to fit the rest of the data. 

# %%
ukesm_sf_vars = ['SFisoprene','SFterpene']
ds_ukesm_sf = ds_ukesm[ukesm_sf_vars].dropna(dim='time').sel(time=slice('2012-01','2018-12'))
ds_ukesm_sf['time'].attrs['timestamp'] = 'mid'

# %%
if ds_ukesm_sf['time'].attrs['timestamp'] =='mid':
    ds_ukesm_sf['time'] = pd.to_datetime(ds_ukesm_sf['time'].values) - datetime.timedelta(hours=1.5)
    ds_ukesm_sf['time'].attrs['timestamp'] = 'start'

# %%
ds_ukesm_sf

# %%
ds_ukesm_dropsf = ds_ukesm.drop(ukesm_sf_vars).dropna(dim='time', )
ds_ukesm_dropsf

# %% [markdown]
# ##### Merge again

# %%
ds_ukesm = xr.merge([ds_ukesm_dropsf,ds_ukesm_sf])
ds_ukesm

# %% [markdown]
# #### Plot 

# %%
for n in num_vars:
    ds_ukesm[n].plot()
    plt.show()

# %%
for n in diam_vars:
    ds_ukesm[n].plot()
    plt.show()

# %%
from bs_fdbck_clean.util.BSOA_datamanip.ukesm import ds2df_ukesm

# %%

df, df_sm = ds2df_ukesm(ds_ukesm,
                        take_daily_median=False,
                        #air_density=air_dens,
                        model_lev_i =model_lev_i
                       )
#df.index = df.reset_index()['time'].apply(fix_echam_time)
df

# %% [markdown] tags=[]
# ### Assuming standard pressure for UKESM (since we don't have this and the effect is marginal)

# %% [markdown]
# Converting by:
#
# \begin{align}
# conc. OA_{amb}=\frac{m_{OA}}{V_{amb}} = & \frac{m_{OA}}{m_{air}} \cdot \frac{m_{air}}{V_{amb}} = w_{OA} \cdot \rho_{amb}
# \end{align}
# and in the same way
# \begin{align}
# conc. OA_{STP}= & \frac{m_{OA}}{m_{air}} \cdot \frac{m_{air}}{V_{STP}} = w_{OA} \cdot \rho_{STP}
# \end{align}
#
# So finally
# \begin{align}
# conc. OA_{STP}= & conc. OA_{amb} \cdot \frac{\rho_{STP}}{\rho_{amb}}
# \end{align}

# %%
from bs_fdbck_clean.util.BSOA_datamanip import pressure_default, standard_air_density,R

# %%
df['T_C']

# %%
df['density'] = 1e5/(R*(df['T_C']+273.15))

# %%
df['density'].plot(marker='.')

# %%
df['ambient2stp_correction_factor'] = (1/df['density'])*standard_air_density

# %%
vl = ['OA','N100','N200','N50','N500']
for v in vl:
    if v in df:
        df = df.rename({v:f'{v}_amb'}, axis=1)
        df[f'{v}_STP'] = df[f'{v}_amb']*df['ambient2stp_correction_factor']
        print(f'converting {v}')

# %%
df['T_C'].plot(marker='.')#.columns#['Temperature'].plot()

# %%
df['N50_STP'].plot()#.columns#['Temperature'].plot()

# %%
df['N50_amb'].plot.hist(bins=100, alpha=.5)#.columns#['Temperature'].plot()
df['N50_STP'].plot.hist(bins=100, alpha=.5)#.columns#['Temperature'].plot()

# %% [markdown]
# ### UKESM Shift time step to start of period to be consistent with measurements and other models

# %%
ind = df.index
df.index = ind - datetime.timedelta(hours=1)

# %% [markdown]
# ### Save result

# %%


_di = {case_name_ukesm:df}
_dism = {case_name_ukesm:df_sm}

dic_df_mod_case['UKESM']= _di.copy()
dic_dfsm_mod_case['UKESM'] = _dism.copy()

# %% [markdown] tags=[]
# ## Adjust NorESM

# %%
ds_noresm = dic_mod_ca['NorESM']['OsloAero_intBVOC_f09_f09_mg17_fssp']

# %%
ds['PS'].sel(station=select_station).plot.hist(bins=100);

# %%
ds_noresm['SOA_A1'].plot.hist(bins=100);

# %%
air_dens = ds_noresm['PS']/(R*ds_noresm['T'])

# %% [markdown]
# ### Run ds2df NorESM

# %%
dic_df = ds2df_inc_preprocessing(dic_mod_ca['NorESM'], 
                                            model_lev_i=model_lev_i, 
                                            air_density=standard_air_density,
                                            select_hours_clouds=False,
                                            mask_cloud_values = False,
#                   from_hour=8,
#                   to_hour=14,
#                   varl_cl=None,
#                   varl_st = None,
          mask_summer = False,
                                            return_summer_median=False, 
                                            take_daily_median=False
                                )


dic_df_mod_case['NorESM'] = dic_df.copy()

# %% [markdown] tags=[]
# ### NorESM needs correcting only for number concentrations

# %% [markdown]
# Converting by:
#
# \begin{align}
# conc. OA_{amb}=\frac{m_{OA}}{V_{amb}} = & \frac{m_{OA}}{m_{air}} \cdot \frac{m_{air}}{V_{amb}} = w_{OA} \cdot \rho_{amb}
# \end{align}
# and in the same way
# \begin{align}
# conc. OA_{STP}= & \frac{m_{OA}}{m_{air}} \cdot \frac{m_{air}}{V_{STP}} = w_{OA} \cdot \rho_{STP}
# \end{align}
#
# So finally
# \begin{align}
# conc. OA_{STP}= & conc. OA_{amb} \cdot \frac{\rho_{STP}}{\rho_{amb}}
# \end{align}

# %%
df = dic_df_mod_case['NorESM'][case_noresm]

# %%
if 'OA' in df.columns:
    df = df.rename({'OA':'OA_STP'}, axis=1)

# %%
pressure = df['PS']
temperature = df['T_C']

# %%
df['density'] = pressure/(R*(temperature+273.15))

# %%
df['density'].plot(marker='.')

# %%
df['ambient2stp_correction_factor'] = (1/df['density'])*standard_air_density

# %%
vl = ['N100','N200','N50','N500']
for v in vl:
    if v in df:
        df = df.rename({v:f'{v}_amb'}, axis=1)
        df[f'{v}_STP'] = df[f'{v}_amb']*df['ambient2stp_correction_factor']
        print(f'converting {v}')

# %%
dic_df_mod_case['NorESM'][case_noresm] = df

# %%
dic_mod_ca['NorESM']['OsloAero_intBVOC_f09_f09_mg17_fssp']['OA'].plot()

# %%
dic_df['OsloAero_intBVOC_f09_f09_mg17_fssp']['N50'].plot()

# %%
dic_df['OsloAero_intBVOC_f09_f09_mg17_fssp']#['50-500'].plot()

# %%
dic_df['OsloAero_intBVOC_f09_f09_mg17_fssp']['N50'].plot()

# %% [markdown]
# ### NorESM Shift time step to start of period to be consistent with measurements and other models

# %%
ind = df.index
df.index = ind - datetime.timedelta(hours=1)

# %% [markdown] tags=[]
# ## ALL MODELS: SHIFT TIME to ATTO which is  european winter time EET UTC-4

# %%
import datetime

# %% tags=[]
for mo in models:
    for ca in di_mod2cases[mo]:
        ind = dic_df_mod_case[mo][ca].index
        dic_df_mod_case[mo][ca].index = ind - datetime.timedelta(hours=4)

# %%
for mo in models:
    for ca in di_mod2cases[mo]:

        print(dic_df_mod_case[mo][ca].index[0:4])

# %% [markdown]
# ### Compute N50-500 etc.

# %%

# %% tags=[]
for mo in models:
    print(mo)
    for ca in di_mod2cases[mo]:
        df = dic_df_mod_case[mo][ca]
        
        for v in ['N50', 'N100', 'N200']:
            _v2 = f'{v}_STP'
            _v2_minus = f'N500_STP'
            if v in df.columns:
                df[v + '-500'] = df[v] - df['N500']
                print(f'computing {v}-500')
            if _v2 in df.columns:
                df[f'{v}-500_STP'] = df[_v2]-df[_v2_minus]
                print(f'computing {v}-500_STP')
                
        dic_df_mod_case[mo][ca] = df
            
        print(v)

# %% [markdown]
# #### NorESM

# %%
rn_dict_echam

# %%
df = dic_df_mod_case['NorESM']['OsloAero_intBVOC_f09_f09_mg17_fssp']
#df = dic_df_mod_case['NorESM']['OsloAero_intBVOC_f19_f19_mg17_fssp']
df['N50_STP'].plot()

# %%
df['T']

# %% [markdown] tags=[]
# ## Write to file:
# %%
for mod in models:
    print(mod)
    dic_df_mod_case[mod]
    for ca in dic_df_mod_case[mod].keys():
        print(mod, ca)
        fn_out = postproc_data/f'{select_station}_station_{mod}_{ca}_ilev{model_lev_i}.csv'
        #else:
        #    fn_out = postproc_data/f'{select_station}_station_{mod}_{ca}.csv'
        print(fn_out)
        dic_df_mod_case[mod][ca].to_csv(fn_out)

# %%
dic_df_mod_case[mod][ca]['OA_STP']#.plot()

# %%
pd.read_csv('/proj/bolinc/users/x_sarbl/analysis/BS-FDBCK/Data/model_station/ATTO/ATTO_station_NorESM_OsloAero_intBVOC_f09_f09_mg17_fssp_ilev-1.csv')

# %%

# %%
